# Generated by Django 2.2.8 on 2022-05-12 11:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0003_department_sort_num'),
    ]

    operations = [
        migrations.AlterField(
            model_name='department',
            name='sort_num',
            field=models.IntegerField(default=1, unique=True, verbose_name='部门排序'),
            preserve_default=False,
        ),
    ]



# -*- coding : utf-8 -*-
"""
@File        : role.py
@Author      : liushuo
@Time        : 2022/5/25 4:20 PM
@Description :
"""
from django.db import transaction

from departments.common.config import BadRequestCode, ServerErrorCode
from departments.utils.error_response import Resp

from departments.utils.role_funcs import (
    create_role,
    get_roles,
    get_role_object,
    update_role,
    delete_role,
    get_role_user,
    add_role_user,
    delete_role_user,
    user_change_role,
    edit_role,
)


class RoleViewSet(object):

    def __init__(self, organization_id: int):
        if not isinstance(organization_id, int):
            raise TypeError(f"organization_id is failed type")
        self.organization_id = organization_id

    @transaction.atomic
    def create(self, data: dict):
        """
        创建角色
        :param data: {"name": "角色名称", "desc": "角色备注"}
        :return:
        """
        data["organization_id"] = self.organization_id
        try:
            return create_role(data)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    def list(self, role_name: str = None):
        """
        获取组织下的角色信息
        :param role_name:
        :return:
        """

        try:
            return get_roles(self.organization_id, role_name)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    def retrieve(self, pk: int):
        """
        获取某一个角色的信息
        :param pk:
        :return:
        """
        return get_role_object(self.organization_id, pk)

    @transaction.atomic
    def update(self, pk: int, data: dict):
        """
        修改某一个角色的信息
        :param pk:
        :param data:
        :return:
        """
        data["organization_id"] = self.organization_id
        try:
            return update_role(pk, data)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    @transaction.atomic
    def delete(self, pk: int):
        """
        删除某一个角色
        :param pk:
        :return:
        """
        return delete_role(self.organization_id, pk)

    @transaction.atomic
    def edit(self, pk: int):
        """
        停用该角色
        :param pk:
        :return:
        """
        return edit_role(self.organization_id, pk)

    @transaction.atomic
    def create_role_user(self, role_id, user_ids: list):
        """
        角色中添加用户（批量添加）
        :param role_id:
        :param user_ids:
        :return:
        """
        try:
            return add_role_user(self.organization_id, role_id, user_ids)
        except Exception as err:
            return Resp(code=BadRequestCode, msg=str(err))

    # def role_user_list(self, role_id, page: int, size: int):
    def role_user_list(self, role_id):
        """
        获取角色对应的用户
        :param role_id:角色的code
        :param page:
        :param size:
        :return:
        """
        """
        部门是根据id查的，为什么角色不可以用id查
        """
        return get_role_user(self.organization_id, role_id)

    @transaction.atomic
    def change(self, user_id: int, role_ids: list):
        """
        用户修改角色
        """
        try:
            return user_change_role(self.organization_id, user_id, role_ids)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    @transaction.atomic
    def delete_role_user(self, role_id, user_ids: list):
        """
        移除对应角色的用户（批量移除）
        :param role_id:
        :param user_ids:
        :return:
        """
        return delete_role_user(self.organization_id, role_id, user_ids)




# -*- coding : utf-8 -*-
"""
@File        : position_funcs.py
@Author      : liushuo
@Time        : 2022/5/30 11:13 AM
@Description :
"""

from django.db import transaction

from departments.common.config import BadRequestCode, ServerErrorCode
from departments.models import Position, UserPosition
from departments.forms.forms import PositionForms
from departments.utils.error_response import Resp


def create_position(data: dict) -> dict:
    """
    创建岗位
    :param data:
    :return:
    """
    # 校验数据的合法性
    checked = PositionForms(data)
    if not checked.is_valid():
        return Resp(code=ServerErrorCode, msg=checked.errors)
    # 查找表里面有没有这个岗位名称，如果有的话，不能添加
    same_position = Position.objects.filter(
        name=data["name"],
        organization_id=data["organization_id"]
    )
    if same_position:
        return Resp(code=ServerErrorCode, msg=f"[{data['name']}]在当前组织中已经存在，请使用其他名称")
    position_instance = Position.objects.create(**data)
    return Resp(data={'id': position_instance.pk})


def get_position(organization_id: int, position_name: str = None):
    """
    获取全部的部门
    :param organization_id:
    :param position_name:
    :return:
    """
    position_items = list()
    # 没有搜索部门名字的时候
    if not position_name:
        instance = Position.objects.filter(
            organization_id=organization_id,
            is_deleted=False
        )
        for i in instance:
            position_items.append({
                "id": i.id,
                "name": i.name,
                "desc": i.desc,
                "created_at": i.created_at
            })
    # 有搜索部门名称的时候
    else:
        instance = Position.objects.get(
            organization_id=organization_id,
            is_deleted=False,
            name__contains=position_name.strip()
        )
        position_items = {
            "id": instance.id,
            "name": instance.name,
            "desc": instance.desc,
            "created_at": instance.created_at
        }
    return Resp(data=position_items)


def retrieve_position(organization_id: int, pk: int):
    """
    获取某个岗位的信息
    :param organization_id:
    :param pk:
    :return:
    """
    try:
        instance = Position.objects.get(
            pk=pk,
            organization_id=organization_id,
            is_deleted=False
        )
        return Resp(
            data={"id": instance.id, "name": instance.name, "desc": instance.desc}
        )
    except Position.DoesNotExist:
        return Resp(code=ServerErrorCode, msg=f"the position can't be found id: {pk}")


@transaction.atomic
def update_position(pk: int, data: dict):
    """
    修改岗位的信息
    :param pk:
    :param data:
    :return:
    """
    """
    先去查询，然后看看是不是创建者，创者者不可编辑，删除，所以不能修改
    """
    # 检查需要修改的数据的合法性
    checked = PositionForms(data)
    if not checked.is_valid():
        return Resp(code=ServerErrorCode, msg=checked.errors)
    organization_id = data["organization_id"]
    try:
        instance = Position.objects.get(
            pk=pk,
            organization_id=organization_id,
            is_deleted=False
        )
        if not instance.is_edited:
            return Resp(code=BadRequestCode, msg="系统岗位，不可修改")
        instance.name = data["name"]
        instance.desc = data["desc"]
        instance.save()
        return Resp(data="修改成功")
    except Exception as err:
        raise ValueError(f"{str(err)} id: {pk}")


def delete_position(organization_id: int, pk: int):
    """
    删除岗位
    :param organization_id:
    :param pk:
    :return:
    """
    instance = Position.objects.get(
        pk=pk,
        organization_id=organization_id,
        is_deleted=False
    )
    if not instance.is_edited:
        return Resp(code=BadRequestCode, msg="系统岗位，不可删除")
    if UserPosition.objects.filter(
            position_id=pk
    ).exists():
        return Resp(data=f"该岗位下有用户，不可删除")

    Position.objects.filter(pk=pk).update(is_deleted=True)  # delete()
    return Resp(data="删除成功")


def add_position_user(organization_id: int, position_id: int, user_ids: list):
    """
    批量添加岗位下的用户
    :param organization_id:
    :param position_id:
    :param user_ids:
    :return:
    """
    """
    先去组织下去查找这个岗位，如果有的话，去添加，
    这个地方用搜索吗？
    下拉框的话，直接就看出有多少岗位来了，就不用查询岗位存不存在了
    """
    if not Position.objects.filter(
            organization_id=organization_id,
            pk=position_id
    ).exists():
        return Resp(code=BadRequestCode, msg="当前组织下没有该岗位")
    position_user_list = [
        UserPosition(position_id=position_id, user_id=user) for user in user_ids
    ]
    try:
        UserPosition.objects.bulk_create(position_user_list)
        return Resp(data="添加成功")
    except Exception as err:
        raise Exception(err)


def get_position_user(organization_id: int, position_id: int):
    """
    获取岗位下的所有用户
    :param organization_id:
    :param position_id:
    :return:
    """
    user_ids = UserPosition.objects.filter(
        position__organization_id=organization_id,
        position__is_deleted=False,
        position_id=position_id,
    ).order_by("id").values_list("user_id", flat=True)
    return Resp(data=list(user_ids))


def delete_position_user(organization_id: int, position_id: int, user_ids: list):
    """
    批量删除岗位下的用户
    :param organization_id:
    :param position_id:
    :param user_ids:
    :return:
    """
    """
    需要查询吗？有没有搜索框？
    """
    if not Position.objects.filter(
        organization_id=organization_id,
        pk=position_id
    ).exists():
        return Resp(code=BadRequestCode, msg="当前组织没有该岗位")
    UserPosition.objects.filter(
        position__organization_id=organization_id,
        position_id=position_id,
        user_id__in=user_ids
    ).delete()
    return Resp(data="删除成功")


@transaction.atomic
def user_change_position(organization_id: int, user_id: int, position_ids: list):
    """
    用户变更岗位
    :param organization_id:
    :param user_id:
    :param position_ids:
    :return:
    """
    try:
        valid_position_count = Position.objects.filter(
            id__in=position_ids,
            organization_id=organization_id
        ).count()
        if not valid_position_count == len(position_ids):
            raise ValueError("参数错误")
        UserPosition.objects.filter(
            position__organization_id=organization_id,
            user_id=user_id
        ).delete()

        changed_instance_list = [UserPosition(position_id=id_, user_id=user_id) for id_ in position_ids]
        UserPosition.objects.bulk_create(changed_instance_list)
        return Resp()
    except Exception as e:
        raise Exception(str(e))





# Generated by Django 2.2.8 on 2022-06-10 15:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0010_auto_20220524_1515'),
    ]

    operations = [
        migrations.CreateModel(
            name='Position',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='最后更新时间')),
                ('name', models.CharField(max_length=32, unique=True, verbose_name='岗位名称')),
                ('desc', models.CharField(blank=True, max_length=255, null=True, verbose_name='岗位描述')),
                ('organization_id', models.IntegerField(verbose_name='组织id')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='是否删除，默认不删除')),
                ('is_edited', models.BooleanField(default=True, verbose_name='是否可以被删除，编辑')),
            ],
            options={
                'verbose_name': '岗位',
                'db_table': 'middle_position',
                'unique_together': {('name', 'organization_id')},
            },
        ),
        migrations.AddField(
            model_name='role',
            name='is_enabled',
            field=models.BooleanField(default=True, verbose_name='是否被启用，默认被启用'),
        ),
        migrations.AlterField(
            model_name='role',
            name='code',
            field=models.CharField(max_length=32, null=True, verbose_name='角色code'),
        ),
        migrations.AlterField(
            model_name='role',
            name='is_deleted',
            field=models.BooleanField(default=False, verbose_name='是否被删除，默认没有被删除'),
        ),
        migrations.AlterField(
            model_name='role',
            name='is_edited',
            field=models.BooleanField(default=True, verbose_name='是否可以被删除，编辑'),
        ),
        migrations.AlterUniqueTogether(
            name='role',
            unique_together={('name', 'organization_id')},
        ),
        migrations.AlterUniqueTogether(
            name='userrole',
            unique_together={('user_id', 'role')},
        ),
        migrations.CreateModel(
            name='UserPosition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='最后更新时间')),
                ('user_id', models.IntegerField(verbose_name='用户id')),
                ('position', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='departments.Position', verbose_name='岗位')),
            ],
            options={
                'verbose_name': '用户岗位关系表',
                'db_table': 'middle_user_position',
                'unique_together': {('user_id', 'position')},
            },
        ),
    ]



# Generated by Django 2.2.8 on 2022-05-12 11:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0002_department_charger'),
    ]

    operations = [
        migrations.AddField(
            model_name='department',
            name='sort_num',
            field=models.IntegerField(null=True, unique=True, verbose_name='部门排序'),
        ),
    ]



# -*- coding : utf-8 -*-
"""
@File        : role_funcs.py
@Author      : liushuo
@Time        : 2022/5/25 4:20 PM
@Description :
"""

from departments.models import Role, UserRole
from departments.forms.forms import RoleForms
from departments.utils.error_response import Resp
from django.db import transaction
from departments.common.config import BadRequestCode, ServerErrorCode


def create_role(data: dict) -> dict:
    """
    新建角色
    :param data:{"name"： "角色名称"， "desc"： "备注"}
    :return:
    """
    # 校验数据
    checked = RoleForms(data)
    # 判断数据合法性
    if not checked.is_valid():
        return Resp(code=ServerErrorCode, msg=checked.errors)
    # 找到最大的排序数
    # if code
    # code = Role.objects.aggregate(num=Max('code'))["num"]
    # data["code"] = int(code) + 1
    # data["is_edited"] = True
    # data["is_deleted"] = True
    # 查找组织里面是否有相同的角色
    same_role = Role.objects.filter(
        name=data["name"],
        organization_id=data["organization_id"]
    ).exists()
    # 如果角色名称存在，返回错
    if same_role:
        return Resp(code=ServerErrorCode, msg=f"[{data['name']}]该角色名称已经存在，请使用其他名称")

    role_instance = Role.objects.create(**data)
    return Resp(data={'id': role_instance.pk})


def get_roles(organization_id: int, role_name: str = None) -> dict:
    """
    通过组织id获取全量角色
    :param organization_id:
    :param role_name: 角色名称，默认为空
    :return:
    """
    items = list()
    # 在没有搜索框的情况下
    if not role_name:
        roles = Role.objects.filter(
            organization_id=organization_id,
            is_deleted=False
        )

        for i in roles:
            items.append({
                "id": i.id,
                "name": i.name,
                "desc": i.desc,
                "created_at": i.created_at
            })
    else:
        roles = Role.objects.filter(
            organization_id=organization_id,
            is_deleted=False,
            name__contains=role_name.strip()
        )
        for item in roles:
            items.append(
                {
                    "id": item.id,
                    "name": item.name,
                    "desc": item.desc,
                    "created_at": item.created_at
                }
            )

    return Resp(data=items)


def get_role_object(organization_id: int, pk: int):
    """
    获取这个组织下，这个角色详情()
    :param organization_id:
    :param pk:
    :return:(返回的时候，id，全部)
    """
    try:
        instance = Role.objects.get(pk=pk, organization_id=organization_id, is_deleted=False)
        return Resp(
            data={"id": instance.id, "name": instance.name, "desc": instance.desc}
        )
    except Role.DoesNotExist:
        return Resp(code=ServerErrorCode, msg=f"the organization can't be found name: {pk}")


def update_role(pk: int, data: dict) -> dict:
    """
    更新角色信息
    :param pk:
    :param data:
    :return:
    """
    # 校验数据
    checked = RoleForms(data)
    if not checked.is_valid():
        return Resp(code=ServerErrorCode, msg=checked.errors)
    organization_id = data["organization_id"]
    try:
        instance = Role.objects.get(pk=pk, organization_id=organization_id, is_deleted=False)
        # instance = Role.objects.get(pk=pk, organization_id=organization_id, is_deleted=False, is_edited=True)
        if not instance.is_edited:
            return Resp(code=BadRequestCode, msg="系统角色，不可修改")
        """
            我需要去查询，他要改的名字是否已经存在，如果角色名称已经存在提示：该角色名称已经存在，请使用其他名称
            如果没有存在，则修改保存数据
        """
        instance.name = data["name"]
        instance.desc = data["desc"]
        instance.save()
        return Resp(data="修改成功")
    except Exception as err:
        raise ValueError(f"{str(err)} id: {pk}")


def delete_role(organization_id: int, pk: int):
    """
    删除角色
    :param organization_id:
    :param pk:
    :return:
    """
    """
    按照id查询，假删除可以，id不会变，
               如果真删除，表中的id就会变---不用管id会不会变，显示出来就好了（根据需求来看把）

    （删除，查询当前角色下还有没有用户，如果没有用户的话，可以删除）
    (查询该角色下还有没有用户的时候，直接按照，组织id和角色的id去查就可以了吧，
    只要角色用户表里面有该角色，就说明该角色下用用户)
    """
    instance = Role.objects.get(pk=pk, organization_id=organization_id, is_deleted=False)
    if not instance.is_edited:
        return Resp(code=BadRequestCode, msg="系统角色，不可删除")
    if UserRole.objects.filter(
            role_id=pk,
    ).exists():
        return ValueError("当前角色下还有用户，无法删除")
    Role.objects.filter(pk=pk).delete()
    return Resp(data="删除成功")


#     Role.objects.filter(
#         pk=pk,
#         organization_id=organization_id,
#         is_edited=True,
#         is_deleted=False
#     # ).delete()
# ).update(is_deleted=True)

def edit_role(organization_id: int, pk: int):
    """
    启用/停用角色
    :param organization_id:
    :param pk:
    :return:
    """
    """
    先去查询这个角色
    判断是不是创建者和管理员，如果是的话，不能操作
    能不能把启用/停用写到一起。
    就写个if，判断是否启用为True，就把它直接赋值成Flase，否则把它赋值成Flase
    """
    instance = Role.objects.get(pk=pk, organization_id=organization_id)
    if not instance.is_edited:
        return Resp(code=BadRequestCode, msg="系统角色，不可操作")
    instance.is_enabled = not instance.is_edited
    instance.save()
    return Resp(data="修改成功")


def add_role_user(organization_id: int, role_id, user_ids: list):
    """
    添加对应角色的用户（批量添加）
    :param organization_id:
    :param role_id:
    :param user_ids:
    :return:
    """
    """
    如果批量给角色添加用户的话，先去角色表查询该角色是否存在
    如果存在，循环需要添加的用户id
    """
    if not Role.objects.filter(
            pk=role_id,
            organization_id=organization_id,
            is_deleted=False
    ).exists():
        return Resp(code=BadRequestCode, msg="该角色不存在")

    role_user_list = [
        UserRole(role_id=role_id, user_id=user) for user in user_ids
    ]
    try:
        UserRole.objects.bulk_create(role_user_list)
    except Exception as err:
        raise Exception(err)
    return Resp(data="添加成功")


def get_role_user(organization_id: int, role_id):
    """
    获取对应角色的用户
    :param organization_id:
    :param role_id:
    :return:
    """
    """
    如果查询该角色的所有用户，需要去角色用户表里面去查询，用户所对应的角色id，根据角色的id查询
    查出来的数据，为该角色对应的用户id
    """
    user_ids = UserRole.objects.filter(
        role__organization_id=organization_id,
        role__is_deleted=False,
        role_id=role_id
    ).order_by("id").values_list("user_id", flat=True)
    return Resp(data=list(user_ids))


def delete_role_user(organization_id: int, role_id, user_ids: list):
    """
    删除对应角色的用户（批量删除）
    :param organization_id:
    :param role_id:
    :param user_ids:
    :return:
    """
    """
    批量删除该角色下的用户，需要先查询该角色是否存在
    角色存在，按照用户的ids去删除
    """
    if not Role.objects.filter(
            organization_id=organization_id,
            pk=role_id
    ).exists():
        return Resp(code=BadRequestCode, msg="当前组织没有该角色")

    UserRole.objects.filter(
        role__organization_id=organization_id,
        role_id=role_id,
        user_id__in=user_ids
    ).delete()
    return Resp(data="删除成功")


def user_change_role(organization_id: int, user_id: int, role_ids: list):
    """
    用户变更角色
    :param organization_id:
    :param user_id:
    :param role_ids:
    :return:
    """
    try:
        valid_role_count = Role.objects.filter(id__in=role_ids, organization_id=organization_id).count()
        if not valid_role_count == len(role_ids):
            raise ValueError("参数错误")
        UserRole.objects.filter(
            role__organization_id=organization_id,
            user_id=user_id
        ).delete()

        changed_instance_list = [UserRole(role_id=id_, user_id=user_id) for id_ in role_ids]
        UserRole.objects.bulk_create(changed_instance_list)
        return Resp()
    except Exception as e:
        raise Exception(str(e))



# Generated by Django 2.2.8 on 2022-06-10 15:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0011_auto_20220610_1549'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='userposition',
            table='middle_user_position_map',
        ),
        migrations.AlterModelTable(
            name='userrole',
            table='middle_user_roles_map',
        ),
    ]



from copy import deepcopy

from django.db import models
from django.contrib.postgres.fields import ArrayField


class BaseModel(models.Model):

    created_at = models.DateTimeField(
        auto_now_add=True,
        verbose_name="创建时间"
    )
    updated_at = models.DateTimeField(
        auto_now=True,
        verbose_name="最后更新时间"
    )

    class Meta:
        abstract = True


class Department(BaseModel):
    """
    部门表
    """
    name = models.CharField(max_length=32, verbose_name="部门名称")
    path = ArrayField(
        models.IntegerField(verbose_name="上级部门ID"),
        null=True,
        verbose_name="所属部门路径"
    )
    organization_id = models.IntegerField(verbose_name="组织id")
    charger_id = models.IntegerField(null=True, verbose_name="负责人ID")
    show_order = models.IntegerField(verbose_name="顺序", default=0)
    is_deleted = models.BooleanField(default=False, verbose_name="部门是否删除, 默认False")

    def get_children(self, department: dict):

        children = department.get("children")
        if not children:
            return
        for i in children:
            # 获取path
            path = deepcopy(i.get("path"))
            path.append(i.get("id"))
            path_str = ",".join([str(j) for j in path])
            # 查询是否有子节点
            child = self.child_map.get(path_str)
            if child:
                i["children"] = child
                # 存在子节点 继续递归调用
                self.get_children(i)
            else:
                i["children"] = []

    def department_tree(self):
        """
        获取树状结构
        @return:
        """
        child = Department.objects.filter(
            path__0=self.id,
            is_deleted=False
        )
        child_map = {}
        # 保存每个节点数据的 一级子节点
        for i in child:
            path_str = ",".join([str(j) for j in i.path])
            i_dict = {
                "id": i.id,
                "name": i.name,
                "charger_id": i.charger_id,
                "path_name": [Department.objects.get(pk=id_).name for id_ in i.path + [i.id]],
                "path": i.path,
                "show_order": i.show_order,
            }
            if path_str not in child_map:
                child_map[path_str] = [i_dict]
            else:
                tmp = child_map[path_str]
                tmp.append(i_dict)
                child_map[path_str] = tmp
        # 准备第一层数据
        res = {
            "id": self.id,
            "name": self.name,
            "charger_id": self.charger_id,
            "path_name": [self.name],
            "path": [],
            "show_order": self.show_order,
            "children": child_map.get(str(self.id), []),
        }
        self.child_map = child_map
        # 递归调用
        self.get_children(res)

        return res

    class Meta:
        db_table = 'department'
        ordering = ['show_order']
        index_together = [
            ["organization_id", "is_deleted"]
        ]
        # unique_together = [
        #     "name", "organization_id"
        # ]


class DepartmentUserMap(BaseModel):
    """
    部门和用户的关系映射表
    """
    department = models.ForeignKey(Department, on_delete=models.CASCADE, verbose_name="部门id")
    user_id = models.IntegerField(verbose_name="用户id")

    class Meta:
        db_table = 'department_user_map'
        unique_together = ["department", "user_id"]


class UserInformation(BaseModel):
    """
    用户信息表
    """
    user_id = models.IntegerField(verbose_name="用户id", unique=True)
    position = models.CharField(max_length=64, verbose_name="职位")
    leader_user_id = models.IntegerField(verbose_name="直属领导id")
    is_deleted = models.BooleanField(default=False, verbose_name="部门是否删除, 默认False")

    class Meta:
        db_table = 'user_information'


class Role(BaseModel):
    """"
    角色表
    """
    name = models.CharField(max_length=32, verbose_name="角色名称")
    desc = models.CharField(max_length=255, null=True, blank=True, verbose_name="角色描述")
    code = models.CharField(max_length=32, null=True, verbose_name="角色code")
    organization_id = models.IntegerField(verbose_name="组织id")
    is_deleted = models.BooleanField(default=False, verbose_name="是否被删除，默认没有被删除")
    is_enabled = models.BooleanField(default=True, verbose_name="是否被启用，默认被启用")
    is_edited = models.BooleanField(default=True, verbose_name="是否可以被删除，编辑")

    def __str__(self):
        return self.name

    class Meta:
        db_table = "middle_roles"
        verbose_name = "角色"
        unique_together = [
            "name", "organization_id"
        ]


class UserRole(BaseModel):
    """
    用户角色表
    """
    user_id = models.IntegerField(
        verbose_name="用户ID"
    )
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        verbose_name="角色"
    )

    class Meta:
        db_table = "middle_user_roles_map"
        verbose_name = "用户角色关系表"
        unique_together = [
            "user_id", "role"
        ]


class Position(BaseModel):
    """
    岗位表
    """
    name = models.CharField(max_length=32, verbose_name="岗位名称")
    desc = models.CharField(max_length=255, null=True, blank=True, verbose_name="岗位描述")
    organization_id = models.IntegerField(verbose_name="组织id")
    is_deleted = models.BooleanField(default=False, verbose_name="是否删除，默认不删除")
    is_edited = models.BooleanField(default=True, verbose_name="是否可以被删除，编辑")

    def __str__(self):
        return self.name

    class Meta:
        db_table = "middle_position"
        verbose_name = "岗位"
        unique_together = [
            "name", "organization_id"
        ]


class UserPosition(BaseModel):
    """
    岗位用户表
    """
    user_id = models.IntegerField(verbose_name="用户id")
    position = models.ForeignKey(
        Position,
        on_delete=models.CASCADE,
        verbose_name="岗位"
    )

    class Meta:
        db_table = "middle_user_position_map"
        verbose_name = "用户岗位关系表"
        unique_together = [
            "user_id", "position"
        ]



# -*- coding: utf-8 -*-
"""
@File        : error_response.py
@Author      : yu wen yang
@Time        : 2022/5/12 1:24 下午
@Description :
"""
from django.http import JsonResponse
from django.core.serializers.json import DjangoJSONEncoder

from departments.common.config import BadRequestCode, Ok


class BadBaseResponse(JsonResponse):
    status_code = BadRequestCode
    default_detail = "bad request"
    default_code = "error"

    def __init__(
            self,
            detail=None,
            code=None,
            encoder=DjangoJSONEncoder,
            safe=True,
            json_dumps_params=None,
            **kwargs,
    ):
        if detail is None:
            detail = self.default_detail
        if code is None:
            code = self.default_code
        super().__init__({'detail': detail, 'code': code}, encoder, safe, json_dumps_params, **kwargs)


class FailedValue(BadBaseResponse):
    status_code = BadRequestCode
    default_detail = "failed value"
    default_code = "failed"


class Resp(dict):
    default_code = Ok
    default_msg = "success"

    def __init__(self, code=None, msg=None, data={}, **kwargs):

        if code is None:
            code = self.default_code
        if msg is None:
            msg = self.default_msg

        dic = {"code": code, "msg": msg, "data": {}}

        dic["data"] = data
        dic.update(kwargs)
        super().__init__(dic, **kwargs)






# Generated by Django 2.2.8 on 2022-05-24 13:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0007_auto_20220520_1447'),
    ]

    operations = [
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='最后更新时间')),
                ('name', models.CharField(max_length=32, verbose_name='角色名称')),
                ('desc', models.CharField(blank=True, max_length=255, null=True, verbose_name='角色描述')),
                ('code', models.CharField(max_length=32, unique=True, verbose_name='角色code')),
                ('organization_id', models.IntegerField(verbose_name='组织id')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='是否已经被删除')),
                ('is_edited', models.BooleanField(default=False, verbose_name='是否可以编辑')),
            ],
            options={
                'verbose_name': '角色',
                'db_table': 'middle_roles',
            },
        ),
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='最后更新时间')),
                ('user_id', models.IntegerField(verbose_name='用户ID')),
                ('role', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='departments.Role', verbose_name='角色')),
            ],
            options={
                'verbose_name': '用户角色关系表',
                'db_table': 'middle_user_roles',
            },
        ),
    ]



# -*- coding : utf-8 -*-
"""
@File        : position.py
@Author      : liushuo
@Time        : 2022/5/30 11:13 AM
@Description :
"""
from django.db import transaction

from departments.common.config import ServerErrorCode, BadRequestCode
from departments.utils.error_response import Resp
from departments.utils.position_funcs import (
    create_position,
    get_position,
    retrieve_position,
    update_position,
    delete_position,
    add_position_user,
    get_position_user,
    delete_position_user,
    user_change_position
)


class PositionViewSet(object):

    def __init__(self, organization_id: int):
        if not isinstance(organization_id, int):
            raise TypeError(f"organization_id is failed type")
        self.organization_id = organization_id

    @transaction.atomic
    def create(self, data: dict):
        """
        创建岗位
        :param data:{"name": "岗位名称", "desc"： "岗位描述"}
        :return:
        """
        data["organization_id"] = self.organization_id
        return create_position(data)
        # return create_position(self.organization_id, data)

    def list(self, position_name: str = None):
        """
        获取全部的角色
        :param position_name:
        :return:
        """
        try:
            return get_position(self.organization_id, position_name)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    def retrieve(self, pk: int):
        """
        获取某个部门的详情
        :param pk:
        :return:
        """
        return retrieve_position(self.organization_id, pk)

    @transaction.atomic
    def update(self, pk: int, data: dict):
        """
        修改岗位信息
        :param pk:
        :param data:
        :return:
        """
        data["organization_id"] = self.organization_id
        try:
            return update_position(pk, data)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    @transaction.atomic
    def delete(self, pk: int):
        """
        删除岗位
        :param pk:
        :return:
        """
        return delete_position(self.organization_id, pk)

    @transaction.atomic
    def create_position_user(self, position_id: int, user_ids: list):
        """
        添加岗位用户（批量添加）
        :param position_id:岗位id
        :param user_ids: 用户id
        :return:
        """
        try:
            return add_position_user(self.organization_id, position_id, user_ids)
        except Exception as err:
            return Resp(code=BadRequestCode, msg=str(err))

    @transaction.atomic
    def change(self, user_id: int, position_ids: list):
        """
        用户变更岗位
        """
        try:
            return user_change_position(self.organization_id, user_id, position_ids)
        except Exception as err:
            return Resp(code=ServerErrorCode, msg=str(err))

    def position_user_list(self, position_id):
        """
        获取岗位下的所有用户
        :param position_id:
        :return:
        """
        return get_position_user(self.organization_id, position_id)

    @transaction.atomic
    def delete_position_user(self, position_id: int, user_ids: list):
        """
        批量删除岗位下的用户
        :param position_id:
        :param user_ids:
        :return:
        """
        return delete_position_user(self.organization_id, position_id, user_ids)





# Generated by Django 2.2.8 on 2022-06-10 16:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0012_auto_20220610_1550'),
    ]

    operations = [
        migrations.AlterField(
            model_name='position',
            name='name',
            field=models.CharField(max_length=32, verbose_name='岗位名称'),
        ),
    ]
